unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, clDownLoader, clDCUtils, clSingleDC, clMultiDC;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    edtRootURL: TEdit;
    btnStart: TButton;
    clDownLoader: TclDownLoader;
    memURLList: TMemo;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnStartClick(Sender: TObject);
    procedure clDownLoaderIsBusyChanged(Sender: TObject);
  private
    FCurrentURLIndex: Integer;
    procedure ProcessURL(const AURL: string);
    procedure ExtractURLS(APage, AURLList: TStrings);
    procedure ProcessNextURL;
    procedure ParsePage(APage, AList: TStrings);
    function GetFullURL(const AURL: string; var AFullURL: string): Boolean;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := not clDownLoader.IsBusy;
  if not CanClose then
  begin
    ShowMessage('The downloading is in progress.');
  end;
end;

procedure TForm1.clDownLoaderIsBusyChanged(Sender: TObject);
var
  List: TStrings;
begin
  if clDownLoader.IsBusy then Exit;
  if FileExists(clDownLoader.LocalFile) then
  begin
    List := TStringList.Create();
    try
      List.LoadFromFile(clDownLoader.LocalFile);
      ExtractURLS(List, memURLList.Lines);
    finally
      List.Free();
    end;
    DeleteFile(clDownLoader.LocalFile);
  end;
  ProcessNextURL();
end;

procedure TForm1.btnStartClick(Sender: TObject);
begin
  if clDownLoader.IsBusy then Exit;
  clDownLoader.LocalFolder := '.\';
  memURLList.Lines.Clear();
  memURLList.Lines.Add(edtRootURL.Text);
  FCurrentURLIndex := -1;
  ProcessNextURL();
end;

procedure TForm1.ProcessNextURL();
begin
  repeat
    Inc(FCurrentURLIndex);
  until (FCurrentURLIndex >= memURLList.Lines.Count) or (Pos('.asp', memURLList.Lines[FCurrentURLIndex]) > 0)
    or (Pos('.htm', memURLList.Lines[FCurrentURLIndex]) > 0);
  if (FCurrentURLIndex < memURLList.Lines.Count) then
  begin
    ProcessURL(memURLList.Lines[FCurrentURLIndex]);
  end else
  begin
    ShowMessage('Process Completed');
  end;
end;

procedure TForm1.ProcessURL(const AURL: string);
begin
  clDownLoader.URL := AURL;
  clDownLoader.Start();
end;

procedure TForm1.ExtractURLS(APage, AURLList: TStrings);
var
  i: Integer;
  List: TStrings;
begin
  List := TStringList.Create();
  try
    ParsePage(APage, List);
    for i := 0 to List.Count - 1 do
    begin
      if (AURLList.IndexOf(List[i]) < 0) then
      begin
        AURLList.Add(List[i]);
      end;
    end;
  finally
    List.Free();
  end;
end;

procedure TForm1.ParsePage(APage, AList: TStrings);
const
  BeginLexem = ' href="';
  EndLexem = '"';
var
  i, j, ParseCount: Integer;
  s, url, fullurl: string;
begin
  for i := 0 to APage.Count - 1 do
  begin
    s := APage[i];
    url := '';
    ParseCount := 0;
    for j := 1 to Length(s) do
    begin
      if (ParseCount = Length(BeginLexem)) then
      begin
        if (s[j] = EndLexem) then
        begin
          if GetFullURL(url, fullurl) and (AList.IndexOf(fullurl) < 0) then
          begin
            AList.Add(fullurl);
          end;
          ParseCount := 0;
          url := '';
        end else
        begin
          url := url + s[j];
        end;
        Continue;
      end;
      if (s[j] = BeginLexem[ParseCount + 1]) then
      begin
        Inc(ParseCount);
      end else
      begin
        ParseCount := 0;
        url := '';
      end;
    end;
  end;
end;

function TForm1.GetFullURL(const AURL: string; var AFullURL: string): Boolean;
var
  URLParser, BaseURLParser: TclURLParser;
  i: Integer;
begin
  Result := False;
  URLParser := TclURLParser.Create();
  BaseURLParser := TclURLParser.Create();
  try
    BaseURLParser.Parse(clDownLoader.URL);
    if (URLParser.Parse(AURL) <> '') and (URLParser.Host <> '') and (Pos('..', URLParser.Host) = 0) then
    begin
      Result := (URLParser.Host = BaseURLParser.Host);
      AFullURL := AURL;
    end else
    if (AURL <> '') then
    begin
      i := 1;
      while (AURL[i] in ['/', '.']) do
      begin
        Inc(i);
      end;
      AFullURL := URLParser.Parse('http://' + BaseURLParser.Host + '/' + Copy(AURL, i, 1000));
      Result := (AFullURL <> '');
    end;
  finally
    BaseURLParser.Free();
    URLParser.Free();
  end;
end;

end.
